//  *****************************************************************************************************************
//  *                                                                                                               *
//  *                                            SpikenzieLabs.com                                                  *
//  *                                                                                                               *
//  *                                          Button 64 Shield Demo                                                *
//  *                                                                                                               *
//  *                                                  MIDI                                                         *
//  *                                                                                                               *
//  *                                                                                                               *
//  *****************************************************************************************************************
//
//  
//  BY: MARK DEMERS 
//  July 2010
//  VERSION 2.0 - Copyright SpikenzieLabs 2010
//
//  www.spikenzielabs.com
// 
// -------------------------------------------------------------------------------------------------------------
//  HOW TO USE THIS DEMO:
//  To use this DEMO sketch, wire up your 64 Button Shield to a set of buttons as described on the 64 Button Shield
//  web page: http://www.spikenzielabs.com/SpikenzieLabs/Button64Shield.html
//
//  Install the 64 Button Shield jumper to the SPI setting, and power-cycle the Ardunio and shield.
//
//  Load this Sketch into your Arduino, and then open the Serial - MIDI Converter and your choice of audio software
//  such as Apple's GarageBand or ABLETON's Live. Set the Serial - MIDI Converter to 115200 bps and the Serial port 
//  to match your Arduino's. Press some buttons, you should hear some music.
//
//  This demo is only for the SPI connection mode.
// 
//  The Serial - MIDI Converter is avaliable here:
//  http://www.spikenzielabs.com/SpikenzieLabs/Serial_MIDI.html
//
// -------------------------------------------------------------------------------------------------------------
//  HOW THIS DEMO WORKS:
//  Three byte serial signals are sent from the Arduino into your music software via the Serial - MIDI software.
//  This is a basic demo, set to MIDI channel 1, with velocity set to full on (127). This sketch could be easily
//  modified in many ways, for example you could read a POT and have variable velocity etc ...
//
//  -------------------------------------------------------------------------------------------------------------
//  LEGAL:
//  This code is provided as is. No guaranties or warranties are given in any form. It is your responsibilty to 
//  determine this codes suitability for your application. Writen permission is required for commercial
//  distribution of this of this code.

//*******************************************************************************************************************
//								                                   VARIABLE INITS
//*******************************************************************************************************************

volatile uint8_t   Button    = 0;                      // Required for 64 Button Shield (SPI Only)

//*******************************************************************************************************************
//								                                           Set-Up
//*******************************************************************************************************************
void setup()
{
  
  Serial.begin(115200);                                // Only used to send data to PC for this DEMO
  
  delay(1000);
  attachInterrupt(0, SPI64B, FALLING);                 // Required for 64 Button Shield (SPI Only)

}

//*******************************************************************************************************************
//								                                         Main Loop
//*******************************************************************************************************************

void loop()
{
  
  if(Button > 0)                              // If Button is > 0, then it was pressed or released (SPI only)
  {
    Serial.print("Button: ");
    if(Button > 128)                          
    {
     Button = Button - 128;
     MIDI_TX(144,Button+28,127);              // NOTE ON
    }else
    {
     MIDI_TX(128,Button+28,127);              // NOTE OFF
    }
    Button = 0;
  }
  
}

//*******************************************************************************************************************
// Required for 64 Button Shield (SPI Only)			                           Functions & Subroutines
//*******************************************************************************************************************
//
// This void is called if the Interrupt 0 is triggered (digital pin 2).
//

void SPI64B()
{
  Button = 0;
  
  volatile uint8_t  val  = 0;
  volatile uint8_t  clk  = 0;
  
  #define  DataInPin   3
  #define  ClkPin      4

  clk = digitalRead(ClkPin);
  while(clk == LOW)
  {
    clk = digitalRead(ClkPin);
  }

  for(volatile int i =0; i<8;i++)
  {   
    val = digitalRead(DataInPin);
    clk = digitalRead(ClkPin); 
    
    while(clk == HIGH)
    {
      clk = digitalRead(ClkPin);
    }

    if(val == HIGH)
    {
      Button = Button +1;
    }
    
    if(i != 7)
    {
      Button = Button << 1;
    }
    else
    {
      break;
    }

    clk = digitalRead(ClkPin);
    
    while(clk == LOW)
    {
      clk = digitalRead(ClkPin);
    }

  }
}    // End  of SPI64B void

//*******************************************************************************************************************
// Transmit MIDI Message			
//*******************************************************************************************************************
void MIDI_TX(unsigned char MESSAGE, unsigned char PITCH, unsigned char VELOCITY) 
{
  Serial.print(MESSAGE);
  Serial.print(PITCH);
  Serial.print(VELOCITY);
}
// -------------------------------------------------------------------------------------------------------------


